//Copyright (c) 1997 The Open Group
//
//Permission to use, copy, modify, and distribute this software and its documentation for
//any purpose and without fee is hereby granted, provided that the above copyright notice
//appears in all copies and that both the copyright notice and this permission notice
//appear in supporting documentation, and that the name of ("TOG") or The Open Group not
//be used in advertising or publicity pertaining to distribution of the software without
//specific, written prior permission.
//
//THE OPEN GROUP DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL
//IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT
//SHALL TOG BE LIABLE FOR ANY SPECIAL, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
//WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN ACTION OF CONTRACT,
//NEGLIGENCE, OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
//PERFORMANCE OF THIS SOFTWARE. 

package ORG.opengroup.jar;
import ORG.opengroup.misc.*;
import java.io.*;
import java.util.*;

/**
 * Represents a manifest of a jar file.
 */
public class Manifest implements Serializable, Cloneable
{
	private Vector sections=new Vector();

	public Manifest()
	{
	}

	/**
	 * Create a manifest from an array of strings.
	 */
	public Manifest(String txt[])
	{
		establish(txt);
	}

	/**
	 * Create a manifest from an array of bytes.
	 */
	public Manifest(byte data[])
	{
		String allData=new String(data);
		MLine ml=new MLine(allData);
		Vector ln=new Vector();
		String s;
		while(ml.hasMoreElements())
			ln.addElement(ml.nextElement());
		int l=ln.size();
		String sa[]=new String[l];
		ln.copyInto(sa);
		establish(sa);
	}

	/**
	 * Create a manifest from an array of strings.
	 */
	private void establish(String txt[])
	{
		int l=txt.length;
		int i=0;
		for(;i<l&&!MLine.empty(txt[i]);++i); //locate empty line
		for(;i<l&&MLine.empty(txt[i]);++i); //locate non-empty line
		while(i<l)
		{
			Vector lnv=new Vector();
			for(;i<l&&!MLine.empty(txt[i]);++i) //process non-empty lines
				lnv.addElement(txt[i]);
			int s=lnv.size();
			String ln[]=new String[s];
			lnv.copyInto(ln);
			sections.addElement(new ManifestSection(ln));
			for(;i<l&&MLine.empty(txt[i]);++i); //locate non-empty line
		}
	}

	/**
	 * Clones the manifest.
	 */
	public synchronized Object clone()
	{
		return new Manifest(getFull());
	}

	/**
	 * Empty the manifest.
	 */
	public synchronized void clear()
	{
		sections=new Vector();
	}

	/**
	 * Returns the array of strings comprising the manifest.
	 */
	public synchronized String get()[]
	{
		Vector lnv=new Vector();
		lnv.addElement("Manifest-Version: 1.0");
		lnv.addElement("");
		int s=sections.size();
		int i;
		for (i=0;i<s;++i)
		{
			ManifestSection ms=(ManifestSection)sections.elementAt(i);
			String ln[]=ms.get();
			int t=ln.length;
			int j;
			for (j=0;j<t;++j)
				lnv.addElement(ln[j]);
			lnv.addElement("");
		}
		String rv[]=new String[lnv.size()];
		lnv.copyInto(rv);
		return rv;
	}

	/**
	 * Returns the array of strings comprising the manifest.
	 * (Line delimiters are included.)
	 */
	public synchronized String getFull()[]
	{
		Vector lnv=new Vector();
		lnv.addElement("Manifest-Version: 1.0");
		lnv.addElement("\n");
		int s=sections.size();
		int i;
		for (i=0;i<s;++i)
		{
			ManifestSection ms=(ManifestSection)sections.elementAt(i);
			String ln[]=ms.getFull();
			int t=ln.length;
			int j;
			for (j=0;j<t;++j)
				lnv.addElement(ln[j]);
			lnv.addElement("\n");
		}
		String rv[]=new String[lnv.size()];
		lnv.copyInto(rv);
		return rv;
	}

	/**
	 * Returns the manifest as an array of bytes.
	 */
	public byte getBytes()[]
	{
		try
		{
			ByteArrayOutputStream baos=new ByteArrayOutputStream();
			OutputStreamWriter osw=new OutputStreamWriter(baos);
			BufferedWriter bw=new BufferedWriter(osw);
			String sa[]=getFull();
			int l=sa.length;
			int i;
			for (i=0;i<l;++i)
			{
				bw.write(sa[i],0,sa[i].length());
				bw.newLine();
			}
			bw.flush();
			return baos.toByteArray();
		} catch (IOException e)
		{
			throw new Error(e.toString());
		}
	}

	/**
	 * Returns the named manifest section. Null is returned if
	 * there is no matching section.
	 */
	public synchronized ManifestSection get(String name)
	{
		int s=sections.size();
		int i;
		for (i=0;i<s;++i)
		{
			ManifestSection ms=(ManifestSection)sections.elementAt(i);
			if (name.equals(ms.get("Name")))
				return (ManifestSection)sections.elementAt(i);
		}
		return null;
	}

	/**
	 * Returns the named manifest section. The section is created if necessary.
	 */
	public synchronized ManifestSection force(String name)
	{
		ManifestSection ms=get(name);
		if (ms!=null)
			return ms;
		ms=new ManifestSection(name);
		add(ms);
		return ms;
	}

	/**
	 * Removes the named manifest section.
	 */
	public synchronized void remove(String name)
	{
		ManifestSection ms=new ManifestSection(name);
		sections.removeElement(ms);
	}

	/**
	 * Adds the manifest section, unless the name starts with
	 * META-INF/ or /META-INF/
	 */
	public synchronized void add(ManifestSection ms)
	{
		String name=ms.get("Name");
		if (name.startsWith("META-INF/") ||
			name.startsWith("/META-INF/"))
			return;
		sections.removeElement(ms);
		sections.addElement(ms);
	}

	/**
	 * Returns an enumeration of the sections of the manifest.
	 */
	public Enumeration sections()
	{
		return sections.elements();
	}
}  
